% Tom Macura 2-13-05 Glue code
% Temporary. Hopefully
%
% OUTPUT GIVEN
%   'sigs_used'     - integers representing which signatures
%                     were found to be the best (collectively)
%                     in classifying the training set.
%   'sigs_used_ind' - doubles recording the signatures'
%                     (corresponding to sigs_used vector) estimated
%                     individual predictive abilities.
%   'sigs_used_col' - doubles recording the signatures'
%                     (corresponding to sigs_used vector) estimated
%                     collective predictive abilities.
%   'sigs_excluded' - sigs that could not be discretized and 
%                     therefore were excluded.
%   'discWalls'     - cell array with bin wall locations
%   'bnet'          - Bayesian Belief Network object
%   'conf_mat'      - Confusion Matrix summarizing results classifier
%                     achieved during training

function [sigs_used, sigs_used_ind, sigs_used_col, sigs_excluded, discWalls, bnet, conf_mat] = ...
	ML_BayesNet_Trainer(contData)

[rows cols] = size(contData);      

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% individually learn wall placements                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sigs_excluded = [];
discWalls = [];

for i = 1:rows-1              % don't involve the class row, (-1)
	discWalls{end+1} = FindDiscretizationWallsFayyadIrani (contData(i,:),contData(end,:));
	
	if length(discWalls{end}) == 0
    	sigs_excluded = [sigs_excluded i];
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% use learned wall placements in discretization                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
discData = ones(size(contData)); 
for i = 1:rows-1
    for j = 1:length(discWalls{i})
        discData(i,:) = discData(i,:) + (contData(i,:) > discWalls{i}(j));
    end
end
discData(end,:) = contData(end,:);

% Uncomment the line below to visualize the feature's discriminating power
% plot_discData(discData([555 end],:));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% find optimal subset of signatures                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[sigs_used, sigs_used_ind, sigs_used_col, conf_mat] = FindSignatureSubset(contData, sigs_excluded, @ConfusionMatrixScore);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% build the Bayes Net (BNET)                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
discData = discData([sigs_used end],:);
[rows cols] = size(discData);
for i=1:length(sigs_used)
	node_sizes(i) = length(discWalls{sigs_used(i)}) + 1;
end
node_sizes(end+1) = length(unique(contData(end,:))); % how many different classes
                                                     % there are
       
% filling the adjacency matrix to make the DAG for a naive bayesian network
dag = false(rows, rows);
dag([1:end-1],end) = true;
                                                   
bnet = mk_bnet(double(dag), double(node_sizes)); % generate bayes net, some datatype issues

for i = 1:rows
    bnet.CPD{i} = tabular_CPD(bnet,i);          % make each node 'tabular' so that it knows 
end                                             % its dealing with discrete data

bnet = learn_params(bnet,double(discData));     % learn the parameters
